<?php

/**
 * Scans for files based on regex patterns and returns an array of paths.
 * 
 * @see hex_get_file_paths()
 */
function _hex_get_file_paths($patterns, $ignore_patterns = NULL, $theme_offset = NULL) {

  list($theme_paths, $plugin_paths, $ignore_all_plugins, $ignore_inactive_plugins) = _hex_prepare_paths();

  // This path will be cut down with each itteration in the following loop
  // to prevent unintentional scanning of unconnected themes and to force
  // a specific order while discovering files.
  $_ignore_paths = $theme_paths;

  $found_files = array();

  foreach (hex_active_themes() as $theme) {
    unset($_ignore_paths[$theme]);
    $ignore_subthemes = array_values($_ignore_paths);

    // Scan and process plug-in owned files first. This is important for
    // cascading order. Theme files have the final hand in overriding files
    // so process theme owned files later.
    foreach (hex_plugin_list_by_owner($theme, 'all') as $plugin) {
      foreach (hex_file_scan($plugin_paths[$plugin], $patterns, $ignore_inactive_plugins, $ignore_patterns) as $file_name => $paths) {
        if (isset($found_files[$file_name])) {
          $found_files[$file_name] = array_merge($found_files[$file_name], $paths);
        }
        else {
          $found_files[$file_name] = $paths;
        }
        $found_files[$file_name] = array_unique($found_files[$file_name]);
      }
    }

    // Scan and process theme owned files. Theme owned files are placed last.
    // Files listed later in the array have more importance in the cascade.
    foreach (hex_file_scan($theme_paths[$theme], $patterns, array_merge($ignore_subthemes, $ignore_all_plugins), $ignore_patterns) as $file_name => $paths) {
      if (isset($found_files[$file_name])) {
        $found_files[$file_name] = array_merge($found_files[$file_name], $paths);
      }
      else {
        $found_files[$file_name] = $paths;
      }
    }
  }

  return $found_files;
}

/**
 * Build paths to files based on regex patterns set in the .info or .pinfo file.
 * 
 * The returned paths are keyed according to the search scope. Possible scopes
 * are "theme", "plugin" or "all".
 * 
 * Patterns set from base themes are inherited into their children. the same
 * applies to plug-in patterns. If the host theme is a parent to another theme
 * the plug-in patterns can influence any child themes. This also depends on
 * the scope of the search.
 */
function _hex_build_file_paths() {

  list($theme_paths, $plugin_paths, $ignore_all_plugins, $ignore_inactive_plugins) = _hex_prepare_paths();

  // This path will be cut down with each itteration in the following loop
  // to prevent unintentional scanning of unconnected themes and to force
  // a specific order while discovering files.
  $_ignore_paths = $theme_paths;

  $found_files = $patterns = array();

  foreach (hex_active_themes() as $theme) {

    unset($_ignore_paths[$theme]);
    $ignore_subthemes = array_values($_ignore_paths);

    // Collect patterns!
    foreach (array('file patterns', 'ignore patterns') as $pattern_type) {

      // First look for plug-in patterns.
      $current_theme_plugin_paths = array();
      foreach (hex_plugin_list_by_owner($theme, 'all') as $plugin) {
        $current_theme_plugin_paths[$plugin] = $plugin_paths[$plugin];
        foreach (hex_plugin_info($plugin, $pattern_type, array(), $theme) as $set => $pattern) {
          // Get search scope (prefixed) from pattern set. General sets defined
          // from plug-ins default to the plug-in scope.
          $search_scopes = array('plugin');
          if (!is_string($set)) {
            $set = 'general';
          }
          elseif (strpos($set, ':') !== FALSE) {
            $search_scopes = explode(':', $set);
            $set = array_pop($search_scopes);
          }
          foreach ($search_scopes as $scope) {
            $patterns[$scope][$set][$pattern_type][] = $pattern;
          }
        }
      }

      // Next, look for theme defined patterns.
      foreach (hex_theme_info($pattern_type, array(), $theme) as $set => $pattern) {
        // Get search scope (prefixed) from pattern set. General sets defined
        // from themes default to the theme scope.
        $search_scopes = array('theme');
        if (!is_string($set)) {
          $set = 'general';
        }
        elseif (strpos($set, ':') !== FALSE) {
          $search_scopes = explode(':', $set);
          $set = array_pop($search_scopes);
        }
        foreach ($search_scopes as $scope) {
          $patterns[$scope][$set][$pattern_type][] = $pattern;
        }
      }
    }

    // Scan with collected patterns!
    foreach ($patterns as $scope => $pattern_sets) {
      foreach ($pattern_sets as $set => $pattern) {
        if (!isset($found_files[$scope][$set])) {
          $found_files[$scope][$set] = array();
        }

        if (!isset($pattern['file patterns'])) {
          continue;
        }
        if (!isset($pattern['ignore patterns'])) {
          $pattern['ignore patterns'] = array();
        }

        // Scan and process plug-in owned files first. This is important for
        // cascading order. Theme files have the final hand in overriding files
        // so process theme owned files later.
        if ($scope == 'all' || $scope == 'plugin') {
          foreach ($current_theme_plugin_paths as $plugin => $path) {
            foreach (hex_file_scan($path, $pattern['file patterns'], $ignore_inactive_plugins, $pattern['ignore patterns']) as $file_name => $paths) {
              if (isset($found_files[$scope][$set][$file_name])) {
                $found_files[$scope][$set][$file_name] = array_merge($found_files[$scope][$set][$file_name], $paths);
              }
              else {
                $found_files[$scope][$set][$file_name] = $paths;
              }
              $found_files[$scope][$set][$file_name] = array_unique($found_files[$scope][$set][$file_name]);

              // Add to theme scope key so it can be selective when called with
              // the scope parameter in hex_path_to_file() and related functions.
              if ($scope == 'all') {
                if (isset($found_files['plugin'][$set][$file_name])) {
                  $found_files['plugin'][$set][$file_name] = array_merge($found_files[$scope][$set][$file_name], $paths);
                }
                else {
                  $found_files['plugin'][$set][$file_name] = $paths;
                }
                $found_files['plugin'][$set][$file_name] = array_unique($found_files[$scope][$set][$file_name]);
              }

            }
          }
        }

        // Scan and process theme owned files. Theme owned files are placed last.
        // Files listed later in the array have more importance in the cascade.
        if ($scope == 'all' || $scope == 'theme') {
          foreach (hex_file_scan($theme_paths[$theme], $pattern['file patterns'], array_merge($ignore_subthemes, $ignore_all_plugins), $pattern['ignore patterns']) as $file_name => $paths) {
            if (isset($found_files[$scope][$set][$file_name])) {
              $found_files[$scope][$set][$file_name] = array_merge($found_files[$scope][$set][$file_name], $paths);
            }
            else {
              $found_files[$scope][$set][$file_name] = $paths;
            }
            // Add to theme scope key so it can be selective when called with
            // the scope parameter in hex_path_to_file() and related functions.
            if ($scope == 'all') {
              if (isset($found_files['theme'][$set][$file_name])) {
                $found_files['theme'][$set][$file_name] = array_merge($found_files[$scope][$set][$file_name], $paths);
              }
              else {
                $found_files['theme'][$set][$file_name] = $paths;
              }
            }
          }
        }

      }
    }
  }

  return $found_files;
}

/**
 * Helper function for hex_get_file_paths() and _hex_build_file_paths().
 * Prepares needed paths.
 */
function _hex_prepare_paths() {
  foreach (list_themes() as $theme => $theme_data) {
    $theme_paths[$theme] = dirname($theme_data->filename);
  }
  foreach (hex_plugin_list('all') as $plugin) {
    $plugin_paths[$plugin] = hex_plugin_path($plugin);
  }
  // Build list of disabled plug-in paths to be ignored.
  // Also build a list of active plug-in paths.
  $plugin_paths_inactive = $plugin_paths;
  foreach (hex_plugin_list() as $plugin) {
    unset($plugin_paths_inactive[$plugin]);
  }
  $ignore_all_plugins = array_values($plugin_paths);
  $ignore_inactive_plugins = array_values($plugin_paths_inactive);

  return array($theme_paths, $plugin_paths, $ignore_all_plugins, $ignore_inactive_plugins);
}

/**
 * Pass a path and regex pattern, get an array of file paths in return.
 * 
 * @param $path
 * @param $patterns
 * @param $ignore_paths
 * @param $ignore_patterns
 */
function hex_file_scan($path, $patterns, $ignore_paths = NULL, $ignore_patterns = NULL) {

  $found_files = array();

  if (is_array($patterns)) {
    $patterns = implode('|', $patterns);
  }
  if (is_array($ignore_patterns)) {
    $ignore_patterns = implode('|', $ignore_patterns);
  }
  if (!isset($ignore_paths)) {
    $ignore_paths = array();
  }

  if ($scan_data = file_scan_directory($path, $patterns)) {
    foreach ($scan_data as $full_path => $file) {
      // Any directory or file prefixed with a tilde will be ignored. This
      // matches the behavior of Photoshop plug-ins and seems like a good way
      // of quickly disabling files or groups of files. Files and directories
      // with leading dots also ignored since they are hidden.
      if (strpos($file->filename, '/~') !== FALSE || strpos($file->filename, '/.') !== FALSE) {
        continue;
      }
      foreach ($ignore_paths as $ignore_path) {
        if (strpos($file->filename, "$ignore_path/") === 0) {
          // Skip to next file.
          continue 2;
        }
      }
      // ereg() used to match Drupal's use in file_scan_directory().
      // Change to preg_match() in Drupal 7.
      if (!(!empty($ignore_patterns) && ereg($ignore_patterns, $file->basename))) {
        $found_files[$file->basename][] = $full_path;
      }
    }
  }

  return $found_files;
}
