<?php

/**
 * Does the following modifications:
 * 
 * - Adds supporting variable preprocess files and files for theme functions
 *   so they are more easily separated and organized. How and when it is
 *   included depends on core version. Versions before 6.16 will add an
 *   additional preprocess function just to load them. For theme functions, it
 *   will change its execution path so it allows preprocessors. See next point.
 * 
 * - Adds preprocess functions for theme functions. See the code comments in
 *   the examples at 'theme-hooks/theme-examples' within the starter theme.
 * 
 * - Stores paths to template suggestions living on the theme layer. Pairing a
 *   base template with its alternate is no longer necessary due to this change.
 * 
 * - Loads of meta data on each theming hook made available to preprocess
 *   functions. Can be seen through '$variables['hook_meta']' or $hook_meta in
 *   template files.
 */
function _hex_theme_registry_alter(&$registry, $auto_includes = array()) {

  // If the modified registry doesn't behave as expected. Uncomment this to
  // inspect all modules which alter the theme registry. The modifications
  // by Hexagon shouldn't be a problem but there could possibly be conflicts.
  // Functions to look into will be named "MODULE_NAME_theme_registry_alter".
  // print var_dump(module_implements('theme_registry_alter'));

  // Collect paths and use them to match the owners for each theming hook.
  $path_to_owner = array();
  foreach (array('module' => module_implements('theme'), 'theme' => hex_active_themes()) as $type => $owners) {
    foreach ($owners as $owner_key) {
      $path_to_owner[drupal_get_path($type, $owner_key)] = $owner_key;
    }
  }
  foreach (hex_plugin_list() as $plugin) {
    $path_to_owner[hex_plugin_path($plugin)] = $plugin;
  }

  // This will be used to find template suggestions.
  $theme_templates = array();
  foreach (hex_get_file_paths('^[a-zA-Z0-9_-]+\.tpl\.php$') as $file_name => $file_paths) {
    $theme_templates[$file_name] = array_pop($file_paths);
  }

  // Find post render callback function.
  $post_render_callback = '';
  foreach (array_reverse(hex_active_themes()) as $theme) {
    if (function_exists($theme . '_post_render_callback')) {
      $post_render_callback = $theme . '_post_render_callback';
      break;
    }
    foreach (hex_plugin_list_by_owner($theme, 'active') as $plugin) {
      if (function_exists($plugin . '_post_render_callback')) {
        $post_render_callback = $plugin . '_post_render_callback';
        break;
      }
    }
  }

  // Gather prefixes in the correct order.
  $prefixes = array('template' => 'template');
  $prefixes += module_list();
  $prefixes += hex_active_themes();
  $prefixes += hex_plugin_list();

  // START REGISTRY EDIT /////////////////////////////////////////////////////

  foreach (array_keys($registry) as $hook) {
    // Reference $hook_data this way for PHP 4.
    $hook_data = &$registry[$hook];

    // If the registered arguments are obviouly wrong, ignore the hook completely.
    if (!empty($hook_data['arguments'])) {
      foreach ($hook_data['arguments'] as $arg => $default) {
        // Numeric argument keys are invalid. Most likely coming from a theme function.
        if (is_numeric($arg)) {
          continue 2;
        }
      }
    }

    // Implemented as...
    $hook_is_function = isset($hook_data['function']) && !isset($hook_data['template']);
    $hook_is_template = !$hook_is_function;
    // Implementation level.
    $type_is_module = $hook_data['type'] == 'module';
    $type_is_theme  = $hook_data['type'] == 'theme';
    $type_is_plugin = $hook_data['type'] == 'plugin';

    // Get the current owner.
    $current_owner  = $path_to_owner[$hook_data['theme path']];
    // And the original implementor of this hook.
    if (isset($path_to_owner[$hook_data['theme paths'][0]])) {
      $original_owner = $path_to_owner[$hook_data['theme paths'][0]];
    }
    else {
      // Cut down the path until the owner is found.
      $path = $hook_data['theme paths'][0];
      while ($pos = strrpos($path, '/')) {
        $path = substr($path, 0, $pos);
        if (isset($path_to_owner[$path])) {
          $original_owner = $path_to_owner[$path];
          break;
        }
      }
    }

    // Useful data about the hook. Used in various places.
    $hook_meta = new stdClass();
    $hook_meta->hook               = $hook;
    $hook_meta->hook_type          = $hook_is_function ? 'function' : 'template';
    $hook_meta->hook_base          = isset($hook_data['original hook']) ? $hook_data['original hook'] : $hook;
    $hook_meta->original_owner     = $original_owner;
    $hook_meta->current_owner      = $current_owner;
    $hook_meta->current_owner_type = $hook_data['type'];

    // PROCESS INCLUDES! /////////////////////////////////////////////////////

    // Set-up all possible prefixes that are associated with this hook.
    $name_prefixes = array($hook);
    // Make sure theming patterns include the originating hook.
    if (isset($hook_data['original hook'])) {
      $name_prefixes[] = $hook_data['original hook'];
    }

    // Add include files based on matching names.
    $include_files = array();
    foreach ($name_prefixes as $prefix) {
      $name_prefix = str_replace('_', '-', $prefix);
      if (isset($hook_data['function']) && isset($auto_includes[$name_prefix . '.func.inc'])) {
        $include_files = array_merge($include_files, $auto_includes[$name_prefix . '.func.inc']);
      }
      if (isset($auto_includes[$name_prefix . '.vars.inc'])) {
        $include_files = array_merge($include_files, $auto_includes[$name_prefix . '.vars.inc']);
      }
    }
    $original_owner_name = str_replace('_', '-', $original_owner);
    if (isset($auto_includes[$original_owner_name . '.theme.inc'])) {
      $include_files = array_merge($include_files, $auto_includes[$original_owner_name . '.theme.inc']);
    }

    // Versions of Drupal Core before 6.16 will not have this set.
    if (!isset($hook_data['include files'])) {
      $hook_data['include files'] = $include_files;
    }
    else {
      $hook_data['include files'] = array_merge($hook_data['include files'], $include_files);
    }

    // Flag for compatibility with includes files.
    // Version of core before 6.16 cannot dynamically include files in theme() calls.
    $includes_compatibility = !empty($include_files) && (float) VERSION <= 6.15;

    // HOOK IS FUNCTION! /////////////////////////////////////////////////////
    if ($hook_is_function) {

      // Clean out any preprocess functions.
      if (isset($hook_data['preprocess functions'])) {
        unset($hook_data['preprocess functions']);
      }

      // This will be used to see if the registry has any changes for modules at the end of this loop.
      $state_for_module_theme_function_hook = $type_is_module ? serialize($hook_data) : FALSE;

      foreach ($prefixes as $prefix) {
        if (function_exists($prefix .'_func_preprocess_'. $hook)) {
          $hook_data['preprocess functions'][] = $prefix .'_func_preprocess_'. $hook;
        }
        if (!empty($hook_data['original hook']) && function_exists($prefix .'_func_preprocess_'. $hook_data['original hook'])) {
          $hook_data['preprocess functions'][] = $prefix .'_func_preprocess_'. $hook_data['original hook'];
        }
      }

      // Allow fine controls in how preprocess functions are listed.
      foreach (array_merge(module_list(), hex_plugin_list(), hex_active_themes()) as $component) {
        if (function_exists($preprocess_alter = $component . '_registry_preprocess_alter')) {
          $_hook_meta = $hook_meta;
          $preprocess_alter($hook_data['preprocess functions'], $_hook_meta);
        }
      }
      // Force uniqueness.
      if (!empty($hook_data['preprocess functions'])) {
        $hook_data['preprocess functions'] = array_unique($hook_data['preprocess functions']);
      }

      // Set a preprocessor just to include any supporting files for older Drupal versions.
      if ($includes_compatibility) {
        array_unshift($hook_data['preprocess functions'], 'hex_process_includes');
      }

      if (!empty($hook_data['preprocess functions'])) {
        array_unshift($hook_data['preprocess functions'], 'hex_theme_func_preprocess_prepare');
        $arguments = &$hook_data['arguments'];
        $c = count($arguments);
        // Buffer 3 argument positions on top of what's already registered.
        // Only needed for theme functions.
        $arguments['unregistered_argument_buffer_' . ++$c]->pinup = 1;
        $arguments['unregistered_argument_buffer_' . ++$c]->pinup = 1;
        $arguments['unregistered_argument_buffer_' . ++$c]->pinup = 1;
        $arguments['hook_meta'] = $hook_meta;
        $arguments['-hex-private-']->function             = $hook_data['function'];
        $arguments['-hex-private-']->post_render_callback = $post_render_callback;
        if ($includes_compatibility) {
          $arguments['-hex-private-']->include_files = $hook_data['include files'];
        }
        unset($hook_data['function']);
        // To prevent notices.
        $hook_data['template'] = 'null';
      }

    }

    // HOOK IS TEMPLATE! /////////////////////////////////////////////////////
    else {

      // Ordered into three phases.
      $variable_processors = array(
        'pre_preprocess' => array(),
        'preprocess'     => $hook_data['preprocess functions'],
        're_process'     => array(),
      );
      foreach (array('pre_preprocess', 'preprocess', 're_process') as $phase) {
        foreach ($prefixes as $prefix) {
          if (function_exists($prefix . '_' . $phase)) {
            $variable_processors[$phase][] = $prefix . '_' . $phase;
          }
          if (function_exists($prefix . '_' . $phase . '_' . $hook)) {
            $variable_processors[$phase][] = $prefix . '_' . $phase . '_' . $hook;
          }
          if (!empty($hook_data['original hook'])) {
            if (function_exists($prefix . '_' . $phase . '_' . $hook_data['original hook'])) {
              $variable_processors[$phase][] = $prefix . '_' . $phase . '_' . $hook_data['original hook'];
            }
          }
        }
      }
      $preprocess_functions = array();
      foreach ($variable_processors as $phase => $preprocessors) {
        $preprocess_functions = array_merge($preprocess_functions, $preprocessors);
      }
      // Force uniqueness and set back to registry.
      $hook_data['preprocess functions'] = array_unique($preprocess_functions);

      // Allow fine controls in how preprocess functions are listed.
      foreach (array_merge(module_list(), hex_plugin_list(), hex_active_themes()) as $component) {
        if (function_exists($preprocess_alter = $component . '_registry_preprocess_alter')) {
          $_hook_meta = $hook_meta;
          $preprocess_alter($hook_data['preprocess functions'], $_hook_meta);
        }
      }
      // Force uniqueness.
      $hook_data['preprocess functions'] = array_unique($hook_data['preprocess functions']);

      // Set a preprocessor just to include any supporting files for older Drupal versions.
      if ($includes_compatibility) {
        array_unshift($hook_data['preprocess functions'], 'hex_process_includes');
      }

      // Discover template paths
      // First create a filter so templates beginning with a similar prefix
      // as the current hook is not mistakenly added.
      $tpl_filter = array();
      foreach (array_keys($registry) as $key) {
        if (strpos($key, $hook) === 0 && $key != $hook) {
          $tpl_filter[] = str_replace('_', '-', $key);
        }
      }

      // Filter for template suggestions to get to their paths.
      foreach ($theme_templates as $name => $template_path) {
        if (preg_match('/^' . str_replace('_', '-', $hook) . '[a-zA-Z0-9_-]+\.tpl\.php$/', $name)) {
          if (!empty($tpl_filter) && preg_match('/^(' . implode('|', $tpl_filter) . ')[a-zA-Z0-9_-]*\.tpl\.php$/', $name)) {
            continue;
          }
          if (!in_array(dirname($template_path), $hook_data['theme paths'])) {
            $hook_data['theme paths'][] = dirname($template_path);
          }
        }
      }

      $arguments = &$hook_data['arguments'];
      $arguments['hook_meta'] = $hook_meta;
      $arguments['-hex-private-']->post_render_callback = $post_render_callback;
      if ($includes_compatibility) {
        $arguments['-hex-private-']->include_files = $hook_data['include files'];
      }
    }

    // This "type" is changed to allow an alternate render function invoked
    // from core's theme() function. When type is "module" and the hook type
    // is a *function*, the theme adding variable preprocessors for the function
    // not hosted in the theme needs this alternate renderer. The alternate
    // render is the phptemplate_render_template() function.
    // 
    // This very selective behavior is to prevent any possible performance
    // degradation for theme functions not hosted in or influenced by the
    // running themes.
    // 
    // Using serialize -a function not meant for this purpose- to compare the
    // multidimentional array but it seems any performance degradation would be
    // noticed only when unserializing very large data sets. Something we aren't
    // doing here.
    if ($hook_is_function && !empty($state_for_module_theme_function_hook) && $state_for_module_theme_function_hook != serialize($hook_data)) {
      $hook_data['type'] = 'module:theme_altered';
    }
    // Hook templates owned by modules are always changed.
    elseif ($hook_is_template && $type_is_module) {
      $hook_data['type'] = 'module:theme_altered';
    }
  }

}

/**
 * Rebuild and save the modified theme registry.
 * 
 * This allows the theme to get the whole scope of the registry unlike
 * HOOK_theme(). It shouldn't break anything and the rest of the theming system
 * will work as it's designed.
 * 
 * This will be called before cores' _theme_load_registry() and allow it to
 * function returning a modified version of the registry. All files should be
 * included to allow discovery of theme functions and variable processors.
 * 
 * The core function _theme_build_registry() will not be invoked. This takes
 * its place.
 */
function _hex_build_registry() {
  global $theme_info, $base_theme_info, $theme_engine;
  $theme = $theme_info;
  $base_theme = $base_theme_info;

  // Prepare files for auto includes.
  $auto_includes = array();
  // Load these files on initialization when the theme registry must be rebuilt.
  // Anything containing a theme function or variable process function must be
  // included here in order for it to be discovered.
  //  - [theme|PLUGIN_NAME].cache.inc
  //  - [PLUGIN_NAME].plug.inc
  //  - [MODULE_NAME|THEME_NAME|PLUGIN_NAME].theme.inc
  //  - [HOOK_NAME].func.inc
  //  - [HOOK_NAME].vars.inc
  //  - template.php
  // Second parameter used to ignore example files.
  foreach (hex_get_file_paths('(^[a-zA-Z0-9_-]+\.(cache|theme|func|vars)\.inc$|^template.php$)', '^(OWNER|HOOK)[a-zA-Z0-9._-]*\.inc$') as $file_name => $file_paths) {
    foreach ($file_paths as $file_path) {
      $auto_includes[$file_name][] = $file_path;
      // Included early to allow function discovery.
      include_once $file_path;
    }
  }


  $cache = array();
  // First, process the theme hooks advertised by modules. This will
  // serve as the basic registry.
  foreach (module_implements('theme') as $module) {
    _theme_process_registry($cache, $module, 'module', $module, drupal_get_path('module', $module));
  }

  // Process each base theme.
  foreach ($base_theme as $base) {
    // If the base theme uses a theme engine, process its hooks.
    $base_path = dirname($base->filename);

    if ($theme_engine) {
      _theme_process_registry($cache, $theme_engine, 'base_theme_engine', $base->name, $base_path);
    }
    foreach (hex_plugin_list_by_owner($base->name) as $plugin) {
      _theme_process_registry($cache, $plugin, 'base_theme_plugin', $plugin, hex_plugin_path($plugin));
    }
    _theme_process_registry($cache, $base->name, 'base_theme', $base->name, $base_path);
  }

  // And then the same thing, but for the theme.
  if ($theme_engine) {
    _theme_process_registry($cache, $theme_engine, 'theme_engine', $theme->name, dirname($theme->filename));
  }
  foreach (hex_plugin_list_by_owner($theme->name) as $plugin) {
    _theme_process_registry($cache, $plugin, 'theme_plugin', $plugin, hex_plugin_path($plugin));
  }
  // Finally, hooks provided by the theme itself.
  _theme_process_registry($cache, $theme->name, 'theme', $theme->name, dirname($theme->filename));

  // Let modules alter the registry
  drupal_alter('theme_registry', $cache);

  // Final alterations, save then set.
  _hex_theme_registry_alter($cache, $auto_includes);
  _theme_save_registry($theme_info, $cache);
  _theme_set_registry($cache);

}
