<?php

/**
 * Returns the path to a file by passing in its name.
 * 
 * @param $name
 *  The file name.
 * @param $group
 *  The group set within the "file pattern" .info entry. "general" is the
 *  default for files found with a pattern not defined in a specific group.
 * @param $scope
 *  Can be 'theme', 'plugin' or 'all'. File discovery is depends on how the
 *  scope is defined in the group prefix.
 */
function hex_path_to_file($name, $group = 'general', $scope = 'theme') {
  $file_paths = _hex_cache('file_paths');
  return isset($file_paths[$scope][$group][$name]) ? array_pop($file_paths[$scope][$group][$name]) : NULL;
}

/**
 * Pass in a group name to get all the files associated with that group.
 * File groups are defined from .info or .pinfo files when defining file patterns.
 * 
 *  file patterns[group name] = ^regex$
 * 
 * There is a 'general' group for files defined without an explicit group name.
 * 
 *  file patterns[] = .+\.(css|js)$
 * 
 * @param $group
 *  The group name. Use 'general' for files not defined with a specific group.
 * @param $scope
 *  Can be 'theme', 'plugin' or 'all'. File discovery is depends on how the
 *  scope is defined in the group prefix.
 * 
 * Both themes and plug-ins:
 *  - file patterns[all:group name] = ^regex$
 * Theme owned files only. No plug-ins:
 *  - file patterns[theme:group name] = ^regex$
 * Plug-in owned files.
 *  - file patterns[plugin:group name] = ^regex$
 */
function hex_group_file_paths($group = 'general', $scope = 'theme') {

  if (!isset($scope)) {
    $scope = 'theme';
  }
  if (!isset($group)) {
    $group = 'general';
  }

  $file_paths = _hex_cache('file_paths');

  $return_paths = array();

  if (isset($file_paths[$scope][$group])) {
    foreach ($file_paths[$scope][$group] as $name => $paths) {
      $return_paths = array_merge($return_paths, $paths);
    }
  }

  return $return_paths;
}

/**
 * Simply returns a list of all defined groups set from file patterns.
 */
function hex_list_file_groups() {
  $list = array();
  foreach (_hex_cache('file_paths') as $groups) {
    foreach (array_keys($groups) as $group) {
      $list[$group] = $group;
    }
  }

  return $list;
}

/**
 * Scans for files based on regex patterns and returns an array of paths.
 * 
 * The scanning starts from base themes and their plug-ins to connected
 * sub-themes which are currently active.
 * 
 * This function should be seldom used as it can place quite a load on the
 * server. Using the cached files for most file searches.
 * 
 * @param $patterns
 *  Regex pattern to be compared with the name of the file. String or an array
 *  of strings are accepted.
 * @param $ignore_patterns
 *  Regex patterns to be compared with the full path including the name of the
 *  file. Any matches will be ignored. String or an array of strings are
 *  accepted.
 * @param $theme_offset
 *  This signals at which base or subtheme to start scanning as it digs further
 *  into connected subthemes.
 * 
 * @see _hex_get_file_paths()
 */
function hex_get_file_paths($patterns, $ignore_patterns = NULL, $theme_offset = NULL) {
  include_once hex_theme_path('hex') . '/includes/build_file_paths.inc';
  return _hex_get_file_paths($patterns, $ignore_patterns, $theme_offset);
}