<?php


/**
 * PRE-PRE-PROCESS -> PRE-PROCESS -> RE-PROCESS (variable processors)
 * ---------------------------------------------------------------------------
 * This theme allows three distinct phases of variable processors to work.
 * Drupal core normally allows only one phase named "preprocess". Hexagon allows
 * the following:
 *  - pre_preprocess      - Early phase.
 *  - preprocess          - Normally allowed in core. Will be invoked as usual.
 *  - re_process          - Late phase.
 * 
 * Then there is the implementation level indicated by the prefix.
 *  - template            - Normally the owner of the hook implements this.
 *  - MODULE_NAME         - Module tapping into the hook. Not the owner.
 *  - ENGINE_NAME_engine  - Implemented by the theme engine itself. Not used.
 *  - ENGINE_NAME         - Engine name used by the theme. Not recommended.
 *  - THEME_NAME          - Theme implementation.
 *  - SUBTHEME_NAME       - Sub-theme implementation.
 *  - PLUGIN_NAME         - Plugin implementation.
 * 
 * "page" hook example in order of execution:
 *  - hex_pre_preprocess
 *  - template_preprocess
 *  - template_preprocess_page
 *  - hex_re_process_page
 *  - smarter_styles_re_process_page (plug-in)
 * 
 * @see http://drupal.org/node/223430
 *  How preprocess functions normally work.
 * 
 * @see http://drupal.org/node/306358
 *  Drupal 7 includes the second phase variable processor but they are named
 *  "process" not "re_process" as it's done here in the late phase. Process is
 *  too commonly used for other functions and conflicts may occur. It will be
 *  safe to change when the theme is upgraded for 7.
 */


/**
 * These variables will be available to all templates, everywhere!
 */
function hex_pre_preprocess(&$variables, $hook) {
  // Initialize id and class attributes.
  if (!isset($variables['id_attribute'])) {
    $variables['id_attribute'] = '';
  }
  $variables['classes_array'][] = $hook;
  // Reference variables so they can be added either way.
  $variables['html_elements']['#attributes']['id']    = &$variables['id_attribute'];
  $variables['html_elements']['#attributes']['class'] = &$variables['classes_array'];
}

/**
 * Prepares arguments for theme functions. We play it safe here and account
 * for unregistered arguments and pass them along.
 */
function hex_theme_func_preprocess_prepare(&$variables, $hook) {

  // Template files not needed for theme functions.
  unset($variables['template_files']);
  // Prepare passed arguments.
  $variables['-hex-private-']->arguments = array();
  foreach ($variables as $key => $var) {
    // Known variables to ignore.
    // $var->pinup checks for unregistered arguments. If it's knocked down,
    // we know it's being used without proper registration. Can be accessed
    // as $variables['unregistered_argument_buffer_ARG_POSITION'].
    if ($key == 'hook_meta' || $key == '-hex-private-' || !empty($var->pinup)) {
      if (strpos($key, 'unregistered_argument_buffer_') === 0) {
        unset($variables[$key]);
      }
      continue;
    }
    // Maintain reference.
    $variables['-hex-private-']->arguments[$key] = &$variables[$key];
  }
}

/**
 * All this does is include supporting files when running Drupal core versions
 * before 6.16. 6.16 contains a fix that automatically includes files from
 * theme() function calls. This is only here to maintain compatibility.
 */
function hex_process_includes(&$variables, $hook) {
  if (!empty($variables['-hex-private-']->include_files)) {
    foreach ($variables['-hex-private-']->include_files as $include) {
      include_once $include;
    }
  }
}

/**
 * Check for template naming suggestions using underscores and add dashes for
 * consistency. This will only run for theming hooks implemented as a template.
 */
function hex_re_process(&$variables, $hook) {
  $suggestions = array();
  foreach ($variables['template_files'] as $suggestion) {
    $suggestions[] = $suggestion;
    // If the underscore is the leading character, this will miss it. It's
    // not a common occurence and doing a strict check of "!== FALSE" has
    // a small performance penalty. This is good enough.
    if (strpos($suggestion, '_')) {
      $suggestions[] = str_replace('_', '-', $suggestion);
    }
  }
  $variables['template_files'] = $suggestions;
}

/**
 * A replacement for theme_render_template(). Used mostly for theme owned hooks.
 * Module owned hooks will use this only in the following two cases:
 * 
 * - Hook is a theme function and the theme uses a variable preprocessor.
 * - Hook has a wrapper involved. This goes for theme functions and templates.
 */
function phptemplate_render_template($template_file, $variables) {

  if (!empty($variables['-hex-private-']->function)) {
    $contents = call_user_func_array($variables['-hex-private-']->function, $variables['-hex-private-']->arguments);
  }
  else {
    extract($variables, EXTR_REFS);  // Extract the variables to a local namespace
    ob_start();                      // Start output buffering
    include "./$template_file";      // Include the template file
    $contents = ob_get_contents();   // Get the contents of the buffer
    ob_end_clean();                  // End buffering and discard
  }

  // Allow preprocessors to set a callback.
  if (!empty($variables['post_render_callback'])) {
    $post_render_callback = $variables['post_render_callback'];
    $contents = $post_render_callback($variables, $contents);
  }
  // Fallback to global post render callback when available.
  elseif (!empty($variables['-hex-private-']->post_render_callback)) {
    $post_render_callback = $variables['-hex-private-']->post_render_callback;
    $contents = $post_render_callback($variables, $contents);
  }

  return $contents;
}
